<?php
/*
  Plugin Name: Cheapest Shipping Rates by Eniture Technology
  Plugin URI: https://eniture.com/products/
  Description: Filters the results from Eniture Technology’s quoting plugins to present a subset of shipping quotes to the site visitor.
  Author: Eniture Technology
  Author URI: https://eniture.com/
  Version: 1.0.0
  Text Domain: eniture-technology
  License: GPL version 2 or later - http://www.eniture.com/
  WC requires at least: 6.4
  WC tested up to: 8.2.2
 */


/**
 * Display cheapest rate in admin side
 */
if (!class_exists('En_Cheapest_Rate')) {

    class En_Cheapest_Rate
    {
        /**
         * Constructor.
         */
        public function __construct()
        {
            add_filter('en_add_cheapest_rate_shipping_addon', [$this, 'en_add_cheapest_rate_shipping_addon'], 10, 1);
            add_filter('woocommerce_package_rates', array($this, 'woocommerce_package_rates'), 999, 2);

            // Settings page
            add_filter('woocommerce_settings_tabs_array', [$this, 'add_settings_tab'], 50);
            add_action('woocommerce_settings_tabs_en_cheapest_rate', [$this, 'settings_tab']);
            add_action('woocommerce_update_options_en_cheapest_rate', [$this, 'update_settings']);
            add_filter('plugin_action_links', 'en_cheapest_rate_sttings_link', 10, 5);
            add_action('admin_enqueue_scripts', array($this, 'en_cheapest_rate_script'));
            add_action('wp_ajax_nopriv_en_wd_save_selected_plugins', array($this, 'save_selected_plugins_ajax'));
            add_action('wp_ajax_en_wd_save_selected_plugins', array($this, 'save_selected_plugins_ajax'));
        }

        /**
         * Load script
         */
        public function en_cheapest_rate_script()
        {
            wp_enqueue_script('en_cheapest_rate_script', plugin_dir_url(__FILE__) . 'assets/js/script.js', array(), '1.0.0');
            wp_localize_script('en_cheapest_rate_script', 'en_cheapest_rate_script', array(
                'en_cheapest_rate_url' => plugins_url()
            ));
            wp_localize_script(
                'jptracks',
                'jpTracksAJAX',
                array(
                    'ajaxurl'            => admin_url( 'admin-ajax.php' ),
                    'jpTracksAJAX_nonce' => wp_create_nonce( 'jp-tracks-ajax-nonce' ),
                )
            );
            wp_register_style('en_cheapest_rate_style', plugin_dir_url(__FILE__) . 'assets/css/cheapest-style.css', false, '1.0.0');
            wp_enqueue_style('en_cheapest_rate_style');
        }

        /**
         * Add a new settings tab to the WooCommerce settings tabs array.
         *
         * @param array $settings_tabs Array of WooCommerce setting tabs & their labels, excluding the Subscription tab.
         * @return array $settings_tabs Array of WooCommerce setting tabs & their labels, including the Subscription tab.
         */
        public static function add_settings_tab($settings_tabs)
        {
            $settings_tabs['en_cheapest_rate'] = __('Cheapest Rate', 'woocommerce-settings-cheapest-rate');
            return $settings_tabs;
        }

        /**
         * Uses the WooCommerce admin fields API to output settings via the @see woocommerce_admin_fields() function.
         *
         * @uses woocommerce_admin_fields()
         * @uses self::get_settings()
         */
        public static function settings_tab()
        {
            woocommerce_admin_fields(self::get_settings());
        }

        /**
         * Uses the WooCommerce options API to save settings via the @see woocommerce_update_options() function.
         *
         * @uses woocommerce_update_options()
         * @uses self::get_settings()
         */
        public static function update_settings()
        {
            woocommerce_update_options(self::get_settings());
        }

        /**
         * Get all the settings for this plugin for @return array Array of settings for @see woocommerce_admin_fields() function.
         * @see woocommerce_admin_fields() function.
         *
         */
        public static function get_settings()
        {
            echo '<div class="cheapest_rate_quote_settings">';
            echo '<div class="en-cheapest-rate-heading"></div>';

            $plugins = ['abf' => 'ABF Freight Edition', 'estes' => 'Estes Edition'];
            $default_option = ['cheapest_rate_abf', 'cheapest_rate_estes'];
            $options = [];
            $is_enabled = get_option('enable_cheapest_rate');
            if (empty($is_enabled)) {
                update_option('enable_cheapest_rate', true);
                $is_enabled = 'true';
            }

            $selected_options = get_option('selected_carriers_cheapest_rate');
            $selected_options = !empty($selected_options) ? json_decode($selected_options, true) : $default_option;
            $selected_options = !empty($selected_options) ? implode(',', $selected_options) : '';

            $cheapest_rate_selection = get_option('cheapest_rate_selection');
            $cheapest_rate_selection = empty($cheapest_rate_selection) ? 'standard_commercial_rate' : $cheapest_rate_selection;
            $additional_services = get_option('additional_services_cheapest_rate');
            $additional_services = empty($additional_services) ? 'display_additional_services' : $additional_services;

            foreach ($plugins as $key => $plugin) {
                $options['cheapest_rate_' . $key] = __('LTL Freight Quotes - ' . $plugin, 'woocommerce-settings-cheapest-rate');
            }

            $settings = array(
                'section_title' => array(
                    'name' => __('Settings', 'woocommerce-settings-cheapest-rate'),
                    'type' => 'title',
                    'desc' => '',
                    'id' => 'wc_en_cheapest_rate_section_title'
                ),
                'enable_cheapest_rate' => array(
                    'name' => __("Enable", 'woocommerce-settings-cheapest-rate'),
                    'type' => 'checkbox',
                    'id' => 'enable_cheapest_rate',
                    'class' => 'enable_cheapest_rate',
                ),
                'selected_carriers_cheapest_rate' => array(
                    'name' => __("Select plugins for cheapest shipping rate", 'woocommerce-settings-cheapest-rate'),
                    'type' => 'select',
                    'default' => $default_option,
                    'options' => $options,
                    'id' => 'selected_carriers_cheapest_rate',
                    'class' => 'selected_carriers_cheapest_rate',
                ),
                'cheapest_rate_selection' => array(
                    'name' => __('Cheapest rate selection', 'woocommerce-settings-cheapest-rate'),
                    'type' => 'radio',
                    'options' => array (
                        'standard_commercial_rate' => __('Use the standard commercial rate as the selector.', 'woocommerce-settings-cheapest-rate'),
                        'cheapest_additional_service' => __('Select the cheapest option for each enabled additional service.', 'woocommerce-settings-cheapest-rate')
                    ),
                    'default' => $cheapest_rate_selection,
                    'id' => 'cheapest_rate_selection',
                    'class' => 'cheapest_rate_selection',
                ),
                'additional_services_cheapest_rate' => array(
                    'name' => __('Additional services', 'woocommerce-settings-cheapest-rate'),
                    'type' => 'radio',
                    'options' => array (
                        'display_additional_services' => __('Display the additional services when they are available for any provider.', 'woocommerce-settings-cheapest-rate'),
                        'supress_additional_services' => __('Suppress the additional services when they aren\'t available for all providers.', 'woocommerce-settings-cheapest-rate')
                    ),
                    'default' => $additional_services,
                    'id' => 'additional_services_cheapest_rate',
                    'class' => 'additional_services_cheapest_rate',
                ),
                'selected_options_cheapest_rate' => array(
                    'name' => __('', 'woocommerce-settings-cheapest-rate'),
                    'type' => 'text',
                    'id' => 'selected_options_cheapest_rate',
                    'class' => 'hidden',
                    'value' => $selected_options
                ),
                'enable_disable_cheapest_rate' => array(
                    'name' => __('', 'woocommerce-settings-cheapest-rate'),
                    'type' => 'text',
                    'id' => 'enable_disable_cheapest_rate',
                    'class' => 'hidden',
                    'value' => $is_enabled
                ),
                'section_end' => array(
                    'type' => 'sectionend',
                    'id' => 'wc_en_cheapest_rate_section_end'
                )
            );

            return $settings;
        }

        /**
         * Existance for addon check
         */
        public function en_add_cheapest_rate_shipping_addon($action)
        {
            return true;
        }

        /**
         * Minimum quotes
         * @param array $rates
         * @return array
         */
        function woocommerce_package_rates($rates, $packages)
        {
            $is_enabled = get_option('enable_cheapest_rate');
            if ($is_enabled == 'false' || empty($rates)) {
                return $rates;
            }

            $plugins_arr = ['abf', 'estes'];
            $standard_rates = [];
            $additional_services_rates = [];

            foreach ($rates as $rate_key => $rate) {
                $method_id = $rate->method_id;
                $rate_id = $rate->id;
                $meta_data = $rate->meta_data['en_fdo_meta_data'];
                if (empty($meta_data)) {
                    continue;
                }

                $data = !empty($meta_data) ? json_decode($meta_data, true) : [];
                $shipment_type = !empty($data) && isset($data['shipment']) ? $data['shipment'] : '';
                $data = !empty($data) && isset($data['data'][0]) ? $data['data'][0] : [];
                $plugin_name = '';
                $owned_by_eniture = false;

                $accessorials = $rate->meta_data['accessorials'];
                $accessorials = !empty($accessorials) ? json_decode($accessorials, true) : [];
                $fdo_meta_data_accessorials = isset($data['accessorials']) ? $data['accessorials'] : [];
                $service_key = '';

                // Check accessorials for additional services rates
                if (!empty((array)$accessorials) || !empty($fdo_meta_data_accessorials) || !empty($shipment_type)) {
                    $hat_label = 'HAT';
                    $hat_access = in_array($hat_label, (array)$accessorials) || (isset($fdo_meta_data_accessorials['holdatterminal']) && $fdo_meta_data_accessorials['holdatterminal']) || strpos($rate_id, $hat_label) != false;
                    if ($hat_access) {
                        $service_key = $hat_label;
                    }

                    $lfg_access = in_array('L', (array)$accessorials) || (isset($fdo_meta_data_accessorials['liftgate']) && $fdo_meta_data_accessorials['liftgate']);
                    $resi_access = in_array('R', (array)$accessorials) || (isset($fdo_meta_data_accessorials['residential']) && $fdo_meta_data_accessorials['residential']);

                    if ($lfg_access && $resi_access) {
                        $service_key = 'LR';
                    } elseif ($lfg_access) {
                        $service_key = 'L';
                    } elseif ($resi_access) {
                        $service_key = 'R';
                    }
                }

                if (!empty($data)) {
                    $plugin_name = isset($data['rate']['plugin_name']) ? $data['rate']['plugin_name'] : '';
                    $owned_by_eniture = isset($data['rate']['owned_by']) && $data['rate']['owned_by'] == 'eniture';

                    // If plugin rate is present in plugins array, add it in separate array and remove from the rates
                    if ((in_array($method_id, $plugins_arr) || in_array($plugin_name, $plugins_arr)) && $owned_by_eniture) {
                        $name = in_array($method_id, $plugins_arr) ? $method_id : $plugin_name;

                        if (!empty($service_key)) {
                            $additional_services_rates[$name][$service_key][$rate_key] = $rate;
                        } else {
                            $standard_rates[$name][$rate_key] = $rate;
                        }

                        unset($rates[$rate_key]);
                    }
                }
            }

            $cheapest_rate_plugin_name = '';
            
            // sort standard rates and add cheapest rate in rates array
            if (!empty($standard_rates) && is_array($standard_rates)) {
                $temp_rates = $standard_rates;
                $standard_rates = array_merge(...array_values($temp_rates));
                array_multisort(array_column($standard_rates, 'cost'), SORT_ASC, $standard_rates);

                $cheapest_rate_key = array_keys($standard_rates)[0];
                $rates[$cheapest_rate_key] = $standard_rates[$cheapest_rate_key];

                foreach ($temp_rates as $key => $value) {
                    if (in_array($cheapest_rate_key, array_keys($value))) {
                        $cheapest_rate_plugin_name = $key;
                        break;
                    }
                }
            }

            // Sort, format and add additional services rates in standard rates
            $additional_services_rates = $this->format_additional_services_rates($additional_services_rates, $cheapest_rate_plugin_name);
            if (!empty($additional_services_rates)) {
                array_multisort(array_column($additional_services_rates, 'cost'), SORT_ASC, $additional_services_rates);
                $rates = array_merge($rates, $additional_services_rates);
            }

            return $rates;
        }

        function format_additional_services_rates($additional_services_rates = [], $plugin_name = '')
        {
            if (empty($additional_services_rates)) {
                return [];
            }

            $cheapest_rate_selection = get_option('cheapest_rate_selection');
            $additional_rates = [];
            $final_rates = [];

            if ($cheapest_rate_selection == 'cheapest_additional_service') {
                $additional_services_selection = get_option('additional_services_cheapest_rate');
                $additional_services_codes = ['HAT', 'L', 'R', 'LR', 'H'];

                // Supress additional services
                if ($additional_services_selection == 'supress_additional_services') {
                    if (count($additional_services_rates) > 1) {
                        $add_services_keys = array_keys($additional_services_rates);
                        $service_in_all_rates = [];
    
                        foreach ($additional_services_rates as $value) {
                            // Get all services keys e.g L,HAT,LR
                            $service_keys = array_keys($value);
    
                            foreach ($service_keys as $value) {
                                // Check all the plugins rates for current service
                                $filtered_arr = array_filter($additional_services_rates, function ($v, $k) use($value) {
                                    return in_array($value, array_keys($v));
                                }, ARRAY_FILTER_USE_BOTH);
    
                                // Add the key whose rates are found in all plugins rates
                                if (count($filtered_arr) == count($add_services_keys)) {
                                    array_push($service_in_all_rates, $value);
                                }
                            }
    
                            // Put rates on their respective service index like L, HAT for sorting
                            if (!empty($service_in_all_rates)) {
                                foreach ($service_in_all_rates as $srvc_key) {
                                    foreach ($additional_services_rates as $key => $value) {
                                        if (isset($value[$srvc_key])) {
                                            $additional_rates[$srvc_key][] = $value[$srvc_key];
                                        }
                                    }
                                }
                            }
    
                            break;
                        }
                    }
                } else {
                    foreach ($additional_services_rates as $srvc_key => $srvc_rate) {
                        $service_value_keys = array_keys($srvc_rate);

                        foreach ($service_value_keys as $key => $value) {
                            $additional_rates[$value][] = $srvc_rate[$value];
                        }
                    }
                }

                // Sort and add cheapest rate of each additional service in rates
                if (!empty($additional_rates)) {
                    foreach ($additional_services_codes as $key => $code) {
                        if (isset($additional_rates[$code])) {
                            // Get rates of each additional service and sort them
                            $service_rates = $additional_rates[$code];
                            $service_rates = array_merge(...$service_rates);
                            array_multisort(array_column($service_rates, 'cost'), SORT_ASC, $service_rates);
                            
                            // Get cheapest rate
                            $final_rates[] = array_slice($service_rates, 0, 1);
                        }
                    }
                }

                // Flattern the rates array
                if (!empty($final_rates)) {
                    $final_rates = array_merge(...$final_rates);
                }
            } else {
                $cheapest_additional_rates = isset($additional_services_rates[$plugin_name]) ? $additional_services_rates[$plugin_name] : [];

                if (!empty($cheapest_additional_rates)) {
                    $final_rates = array_merge(...array_values($cheapest_additional_rates));
                }
            }

            return $final_rates;
        }

        function save_selected_plugins_ajax()
        {
            $inputData = $_POST;
            $selected_plugins = isset($inputData['selected_plugins']) ? $inputData['selected_plugins'] : [];
            $enabled = isset($inputData['enabled']) ? $inputData['enabled'] : false;
            $cheapest_rate_selection = isset($inputData['cheapest_rate_selection']) ? $inputData['cheapest_rate_selection'] : '';
            $additional_services = isset($inputData['additional_services']) ? $inputData['additional_services'] : '';
            $response = [];

            if (empty($selected_plugins) || count($selected_plugins) < 2) {
                $response = [
                    'error' => true,
                    'message' => 'Please select at least two plugin options.',
                    'selected_plugins' => $selected_plugins,
                    'enabled' => $enabled
                ];
            } else {
                $selected_plugins = json_encode($selected_plugins);
                update_option('selected_carriers_cheapest_rate', $selected_plugins);
                update_option('enable_cheapest_rate', $enabled);
                update_option('cheapest_rate_selection', $cheapest_rate_selection);
                update_option('additional_services_cheapest_rate', $additional_services);

                $response = [
                    'error' => false,
                    'message' => 'Your settings have been saved.',
                    'selected_plugins' => $selected_plugins,
                    'enabled' => $enabled,
                    'cheapest_rate_selection' => $cheapest_rate_selection,
                    'additional_services' => $additional_services
                ];
            }

            echo json_encode($response);
            exit();
        }
    }

    /**
     * update actions links
     * @staticvar $plugin
     * @param $actions
     * @param $plugin_file
     * @return Links
     */
    function en_cheapest_rate_sttings_link($actions, $plugin_file)
    {
        static $plugin;

        if (!isset($plugin))
            $plugin = plugin_basename(__FILE__);
        if ($plugin == $plugin_file) {
            $settings_link = wp_kses_post(sprintf(__('<a href="admin.php?page=wc-settings&tab=en_cheapest_rate">' . __('Settings', 'General') . '</a>')));
            $support_link = wp_kses_post(sprintf(__('<a href="https://eniture.com/support/" target="_blank">Support</a>')));
            $settings = array('settings' => $settings_link);
            $site_link = array('support' => $support_link);
            $actions = array_merge($settings, $actions);
            $actions = array_merge($site_link, $actions);
        }

        return $actions;
    }

    new En_Cheapest_Rate();
}